@@ -1,7 +1,9 @@
 module Language.JavaScript.Parser
        (
          PA.parse
+       , PA.parseModule
        , PA.readJs
+       , PA.readJsModule
        , PA.parseFile
        , PA.parseFileUtf8
        , PA.showStripped
@@ -40,5 +42,3 @@ import Language.JavaScript.Parser.SrcLocation
 import Language.JavaScript.Pretty.Printer
 
 -- EOF
-
-
