@@ -21,12 +21,14 @@ import Language.JavaScript.Parser.SrcLocation
 data AlexUserState = AlexUserState
     { previousToken :: !Token   -- ^the previous token
     , comment :: [Token]        -- ^the previous comment, if any
+    , inTemplate :: Bool        -- ^whether the parser is expecting template characters
     }
 
 alexInitUserState :: AlexUserState
 alexInitUserState = AlexUserState
     { previousToken = initToken
     , comment = []
+    , inTemplate = False
     }
 
 initToken :: Token
